@@ -13,158 +13,191 @@ void copy_warning(){
   Rf_warning("x is not a double vector and has been copied, it will not be replaced by reference.\n\tEnsure the result is assigned to an object if used in further calculations\n\te.g. `x <- set_log(x)`");
 }
 
-// Source: https://stackoverflow.com/questions/32746523/ieee-754-compliant-round-half-to-even
 double round_nearest_even(double x){
   x -= std::remainder(x, 1.0);
   return x;
 }
 
+SEXP check_transform_altrep(SEXP x){
+  if (ALTREP(x)){
+    Rf_warning("Cannot update an ALTREP by reference, a copy has been made. \n\tEnsure the result is assigned to an object if used in further calculations\n\te.g. `x <- set_abs(x)`");
+    return altrep_materialise(x);
+  } else {
+    return x;
+  }
+}
+
+
+// Convert integer vector to plain double vector
+
+SEXP convert_int_to_real(SEXP x){
+  int *p_x = INTEGER(x);
+  R_xlen_t n = Rf_xlength(x);
+  SEXP out = Rf_protect(Rf_allocVector(REALSXP, n));
+  double *p_out = REAL(out);
+  for (int i = 0; i < n; ++i){
+    p_out[i] = p_x[i] != NA_INTEGER ? (double)(p_x[i]) : NA_REAL;
+  }
+  Rf_unprotect(1);
+  return out;
+}
+
 [[cpp11::register]]
 SEXP cpp_set_abs(SEXP x){
   cpp_check_numeric(x);
-  R_xlen_t n = Rf_xlength(x);
+  SEXP out = Rf_protect(check_transform_altrep(x));
+  R_xlen_t n = Rf_xlength(out);
   int n_cores = n >= 100000 ? num_cores() : 1;
-  switch (TYPEOF(x)){
+  switch (TYPEOF(out)){
   case INTSXP: {
-    int *p_x = INTEGER(x);
+    int *p_out = INTEGER(out);
     if (n_cores > 1){
       OMP_PARALLEL_FOR_SIMD
       for (R_xlen_t i = 0; i < n; ++i) {
-        p_x[i] = p_x[i] == NA_INTEGER ? p_x[i] : std::abs(p_x[i]);
+        p_out[i] = p_out[i] == NA_INTEGER ? p_out[i] : std::abs(p_out[i]);
       }
     } else {
       OMP_FOR_SIMD
       for (R_xlen_t i = 0; i < n; ++i) {
-        p_x[i] = p_x[i] == NA_INTEGER ? p_x[i] : std::abs(p_x[i]);
+        p_out[i] = p_out[i] == NA_INTEGER ? p_out[i] : std::abs(p_out[i]);
       }
     }
     break;
   }
   case REALSXP: {
-    double *p_x = REAL(x);
+    double *p_out = REAL(out);
     if (n_cores > 1){
       OMP_PARALLEL_FOR_SIMD
       for (R_xlen_t i = 0; i < n; ++i) {
-        p_x[i] = p_x[i] != p_x[i] ? p_x[i] : std::fabs(p_x[i]);
+        p_out[i] = p_out[i] != p_out[i] ? p_out[i] : std::fabs(p_out[i]);
       }
     }
     else {
       OMP_FOR_SIMD
       for (R_xlen_t i = 0; i < n; ++i) {
-        p_x[i] = p_x[i] != p_x[i] ? p_x[i] : std::fabs(p_x[i]);
+        p_out[i] = p_out[i] != p_out[i] ? p_out[i] : std::fabs(p_out[i]);
       }
     }
     break;
   }
   }
-  return x;
+  Rf_unprotect(1);
+  return out;
 }
 
 [[cpp11::register]]
 SEXP cpp_set_floor(SEXP x){
   cpp_check_numeric(x);
-  R_xlen_t n = Rf_xlength(x);
+  SEXP out = Rf_protect(check_transform_altrep(x));
+  R_xlen_t n = Rf_xlength(out);
   int n_cores = n >= 100000 ? num_cores() : 1;
-  if (Rf_isReal(x)){
-    double *p_x = REAL(x);
+  if (Rf_isReal(out)){
+    double *p_out = REAL(out);
     if (n_cores > 1){
       OMP_PARALLEL_FOR_SIMD
       for (R_xlen_t i = 0; i < n; ++i) {
-        p_x[i] = (p_x[i] == p_x[i]) ? std::floor(p_x[i]) : p_x[i];
+        p_out[i] = (p_out[i] == p_out[i]) ? std::floor(p_out[i]) : p_out[i];
       }
     } else {
       OMP_FOR_SIMD
       for (R_xlen_t i = 0; i < n; ++i) {
-        p_x[i] = (p_x[i] == p_x[i]) ? std::floor(p_x[i]) : p_x[i];
+        p_out[i] = (p_out[i] == p_out[i]) ? std::floor(p_out[i]) : p_out[i];
       }
     }
   }
-  return x;
+  Rf_unprotect(1);
+  return out;
 }
 
 [[cpp11::register]]
 SEXP cpp_set_ceiling(SEXP x){
   cpp_check_numeric(x);
+  SEXP out = Rf_protect(check_transform_altrep(x));
   R_xlen_t n = Rf_xlength(x);
   int n_cores = n >= 100000 ? num_cores() : 1;
-  if (Rf_isReal(x)){
-    double *p_x = REAL(x);
+  if (Rf_isReal(out)){
+    double *p_out = REAL(out);
     if (n_cores > 1){
       OMP_PARALLEL_FOR_SIMD
       for (R_xlen_t i = 0; i < n; ++i) {
-        p_x[i] = (p_x[i] == p_x[i]) ? std::ceil(p_x[i]) : p_x[i];
+        p_out[i] = (p_out[i] == p_out[i]) ? std::ceil(p_out[i]) : p_out[i];
       }
     } else {
       OMP_FOR_SIMD
       for (R_xlen_t i = 0; i < n; ++i) {
-        p_x[i] = (p_x[i] == p_x[i]) ? std::ceil(p_x[i]) : p_x[i];
+        p_out[i] = (p_out[i] == p_out[i]) ? std::ceil(p_out[i]) : p_out[i];
       }
     }
   }
-  return x;
+  Rf_unprotect(1);
+  return out;
 }
 
 [[cpp11::register]]
 SEXP cpp_set_trunc(SEXP x){
   cpp_check_numeric(x);
-  R_xlen_t n = Rf_xlength(x);
+  SEXP out = Rf_protect(check_transform_altrep(x));
+  R_xlen_t n = Rf_xlength(out);
   int n_cores = n >= 100000 ? num_cores() : 1;
-  if (Rf_isReal(x)){
-    double *p_x = REAL(x);
+  if (Rf_isReal(out)){
+    double *p_out = REAL(out);
     if (n_cores > 1){
       OMP_PARALLEL_FOR_SIMD
       for (R_xlen_t i = 0; i < n; ++i) {
-        p_x[i] = (p_x[i] == p_x[i]) ? std::trunc(p_x[i]) : p_x[i];
+        p_out[i] = (p_out[i] == p_out[i]) ? std::trunc(p_out[i]) : p_out[i];
       }
     } else {
       OMP_FOR_SIMD
       for (R_xlen_t i = 0; i < n; ++i) {
-        p_x[i] = (p_x[i] == p_x[i]) ? std::trunc(p_x[i]) : p_x[i];
+        p_out[i] = (p_out[i] == p_out[i]) ? std::trunc(p_out[i]) : p_out[i];
       }
     }
   }
-  return x;
+  Rf_unprotect(1);
+  return out;
 }
 
 [[cpp11::register]]
 SEXP cpp_set_change_sign(SEXP x){
   cpp_check_numeric(x);
-  R_xlen_t n = Rf_xlength(x);
+  SEXP out = Rf_protect(check_transform_altrep(x));
+  R_xlen_t n = Rf_xlength(out);
   int n_cores = n >= 100000 ? num_cores() : 1;
-  switch (TYPEOF(x)){
+  switch (TYPEOF(out)){
   case INTSXP: {
-    int *p_x = INTEGER(x);
+    int *p_out = INTEGER(out);
     if (n_cores > 1){
       OMP_PARALLEL_FOR_SIMD
       for (R_xlen_t i = 0; i < n; ++i) {
-        p_x[i] = (p_x[i] == NA_INTEGER) ? p_x[i] : -p_x[i];
+        p_out[i] = (p_out[i] == NA_INTEGER) ? p_out[i] : -p_out[i];
       }
     } else {
       OMP_FOR_SIMD
       for (R_xlen_t i = 0; i < n; ++i) {
-        p_x[i] = (p_x[i] == NA_INTEGER) ? p_x[i] : -p_x[i];
+        p_out[i] = (p_out[i] == NA_INTEGER) ? p_out[i] : -p_out[i];
       }
     }
     break;
   }
   case REALSXP: {
-    double *p_x = REAL(x);
+    double *p_out = REAL(out);
     if (n_cores > 1){
       int n_cores = num_cores();
       OMP_PARALLEL_FOR_SIMD
       for (R_xlen_t i = 0; i < n; ++i) {
-        p_x[i] = (p_x[i] == p_x[i]) ? -p_x[i] : p_x[i];
+        p_out[i] = (p_out[i] == p_out[i]) ? -p_out[i] : p_out[i];
       }
     } else {
       OMP_FOR_SIMD
       for (R_xlen_t i = 0; i < n; ++i) {
-        p_x[i] = (p_x[i] == p_x[i]) ? -p_x[i] : p_x[i];
+        p_out[i] = (p_out[i] == p_out[i]) ? -p_out[i] : p_out[i];
       }
     }
     break;
   }
   }
-  return x;
+  Rf_unprotect(1);
+  return out;
 }
 
 [[cpp11::register]]
@@ -172,22 +205,27 @@ SEXP cpp_set_exp(SEXP x){
   cpp_check_numeric(x);
   R_xlen_t n = Rf_xlength(x);
   int n_cores = n >= 100000 ? num_cores() : 1;
-  if (!Rf_isReal(x)) copy_warning();
-  Rf_protect(x = Rf_coerceVector(x, REALSXP));
-  double *p_x = REAL(x);
+  SEXP out;
+  if (!Rf_isReal(x)){
+    copy_warning();
+    out = Rf_protect(convert_int_to_real(x));
+  } else {
+    out = Rf_protect(check_transform_altrep(x));
+  }
+  double *p_out = REAL(out);
   if (n_cores > 1){
     OMP_PARALLEL_FOR_SIMD
     for (R_xlen_t i = 0; i < n; ++i) {
-      p_x[i] = (p_x[i] == p_x[i]) ? std::exp(p_x[i]) : p_x[i];
+      p_out[i] = (p_out[i] == p_out[i]) ? std::exp(p_out[i]) : p_out[i];
     }
   } else {
     OMP_FOR_SIMD
     for (R_xlen_t i = 0; i < n; ++i) {
-      p_x[i] = (p_x[i] == p_x[i]) ? std::exp(p_x[i]) : p_x[i];
+      p_out[i] = (p_out[i] == p_out[i]) ? std::exp(p_out[i]) : p_out[i];
     }
   }
   Rf_unprotect(1);
-  return x;
+  return out;
 }
 
 [[cpp11::register]]
@@ -195,9 +233,14 @@ SEXP cpp_set_sqrt(SEXP x){
   cpp_check_numeric(x);
   R_xlen_t n = Rf_xlength(x);
   int n_cores = n >= 100000 ? num_cores() : 1;
-  if (!Rf_isReal(x)) copy_warning();
-  Rf_protect(x = Rf_coerceVector(x, REALSXP));
-  double *p_x = REAL(x);
+  SEXP out;
+  if (!Rf_isReal(x)){
+    copy_warning();
+    out = Rf_protect(convert_int_to_real(x));
+  } else {
+    out = Rf_protect(check_transform_altrep(x));
+  }
+  double *p_x = REAL(out);
   if (n_cores > 1){
     OMP_PARALLEL_FOR_SIMD
     for (R_xlen_t i = 0; i < n; ++i) {
@@ -210,27 +253,29 @@ SEXP cpp_set_sqrt(SEXP x){
     }
   }
   Rf_unprotect(1);
-  return x;
+  return out;
 }
 
 [[cpp11::register]]
 SEXP cpp_set_add(SEXP x, SEXP y){
   cpp_check_numeric(x);
   cpp_check_numeric(y);
-  R_xlen_t xn = Rf_xlength(x);
+  int n_protections = 0;
+  SEXP out = Rf_protect(check_transform_altrep(x));
+  ++n_protections;
+  R_xlen_t xn = Rf_xlength(out);
   R_xlen_t yn = Rf_xlength(y);
   int n_cores = xn >= 100000 ? num_cores() : 1;
   if (yn > xn){
     Rf_error("length(y) must be <= length(x)");
   }
-  int n_protections = 0;
-  switch (TYPEOF(x)){
+  switch (TYPEOF(out)){
   case LGLSXP:
   case INTSXP: {
     switch (TYPEOF(y)){
   case LGLSXP:
   case INTSXP: {
-    int *p_x = INTEGER(x);
+    int *p_x = INTEGER(out);
     int *p_y = INTEGER(y);
 #pragma omp parallel for simd num_threads(n_cores) if (n_cores > 1)
     for (R_xlen_t i = 0; i < xn; ++i) {
@@ -242,9 +287,9 @@ SEXP cpp_set_add(SEXP x, SEXP y){
   }
   case REALSXP: {
     copy_warning();
-    Rf_protect(x = Rf_coerceVector(x, REALSXP));
+    Rf_protect(out = Rf_coerceVector(out, REALSXP));
     ++n_protections;
-    double *p_x = REAL(x);
+    double *p_x = REAL(out);
     double *p_y = REAL(y);
 #pragma omp parallel for simd num_threads(n_cores) if (n_cores > 1)
     for (R_xlen_t i = 0; i < xn; ++i) {
@@ -261,9 +306,7 @@ SEXP cpp_set_add(SEXP x, SEXP y){
     switch (TYPEOF(y)){
   case LGLSXP:
   case INTSXP: {
-    // Rf_protect(y = Rf_coerceVector(y, REALSXP));
-    // ++n_protections;
-    double *p_x = REAL(x);
+    double *p_x = REAL(out);
     int *p_y = INTEGER(y);
 #pragma omp parallel for simd num_threads(n_cores) if (n_cores > 1)
     for (R_xlen_t i = 0; i < xn; ++i) {
@@ -274,7 +317,7 @@ SEXP cpp_set_add(SEXP x, SEXP y){
     break;
   }
   case REALSXP: {
-    double *p_x = REAL(x);
+    double *p_x = REAL(out);
     double *p_y = REAL(y);
 #pragma omp parallel for simd num_threads(n_cores) if (n_cores > 1)
     for (R_xlen_t i = 0; i < xn; ++i) {
@@ -288,27 +331,29 @@ SEXP cpp_set_add(SEXP x, SEXP y){
   }
   }
   Rf_unprotect(n_protections);
-  return x;
+  return out;
 }
 
 [[cpp11::register]]
 SEXP cpp_set_subtract(SEXP x, SEXP y){
   cpp_check_numeric(x);
   cpp_check_numeric(y);
-  R_xlen_t xn = Rf_xlength(x);
+  int n_protections = 0;
+  SEXP out = Rf_protect(check_transform_altrep(x));
+  ++n_protections;
+  R_xlen_t xn = Rf_xlength(out);
   R_xlen_t yn = Rf_xlength(y);
   int n_cores = xn >= 100000 ? num_cores() : 1;
   if (yn > xn){
     Rf_error("length(y) must be <= length(x)");
   }
-  int n_protections = 0;
-  switch (TYPEOF(x)){
+  switch (TYPEOF(out)){
   case LGLSXP:
   case INTSXP: {
     switch (TYPEOF(y)){
   case LGLSXP:
   case INTSXP: {
-    int *p_x = INTEGER(x);
+    int *p_x = INTEGER(out);
     int *p_y = INTEGER(y);
 #pragma omp parallel for simd num_threads(n_cores) if (n_cores > 1)
     for (R_xlen_t i = 0; i < xn; ++i) {
@@ -320,9 +365,9 @@ SEXP cpp_set_subtract(SEXP x, SEXP y){
   }
   case REALSXP: {
     copy_warning();
-    Rf_protect(x = Rf_coerceVector(x, REALSXP));
+    Rf_protect(out = Rf_coerceVector(out, REALSXP));
     ++n_protections;
-    double *p_x = REAL(x);
+    double *p_x = REAL(out);
     double *p_y = REAL(y);
 #pragma omp parallel for simd num_threads(n_cores) if (n_cores > 1)
     for (R_xlen_t i = 0; i < xn; ++i) {
@@ -339,9 +384,7 @@ SEXP cpp_set_subtract(SEXP x, SEXP y){
     switch (TYPEOF(y)){
   case LGLSXP:
   case INTSXP: {
-    // Rf_protect(y = Rf_coerceVector(y, REALSXP));
-    // ++n_protections;
-    double *p_x = REAL(x);
+    double *p_x = REAL(out);
     int *p_y = INTEGER(y);
 #pragma omp parallel for simd num_threads(n_cores) if (n_cores > 1)
     for (R_xlen_t i = 0; i < xn; ++i) {
@@ -352,7 +395,7 @@ SEXP cpp_set_subtract(SEXP x, SEXP y){
     break;
   }
   case REALSXP: {
-    double *p_x = REAL(x);
+    double *p_x = REAL(out);
     double *p_y = REAL(y);
 #pragma omp parallel for simd num_threads(n_cores) if (n_cores > 1)
     for (R_xlen_t i = 0; i < xn; ++i) {
@@ -366,27 +409,29 @@ SEXP cpp_set_subtract(SEXP x, SEXP y){
   }
   }
   Rf_unprotect(n_protections);
-  return x;
+  return out;
 }
 
 [[cpp11::register]]
 SEXP cpp_set_multiply(SEXP x, SEXP y){
   cpp_check_numeric(x);
   cpp_check_numeric(y);
-  R_xlen_t xn = Rf_xlength(x);
+  int n_protections = 0;
+  SEXP out = Rf_protect(check_transform_altrep(x));
+  ++n_protections;
+  R_xlen_t xn = Rf_xlength(out);
   R_xlen_t yn = Rf_xlength(y);
   int n_cores = xn >= 100000 ? num_cores() : 1;
   if (yn > xn){
     Rf_error("length(y) must be <= length(x)");
   }
-  int n_protections = 0;
-  switch (TYPEOF(x)){
+  switch (TYPEOF(out)){
   case LGLSXP:
   case INTSXP: {
     switch (TYPEOF(y)){
   case LGLSXP:
   case INTSXP: {
-    int *p_x = INTEGER(x);
+    int *p_x = INTEGER(out);
     int *p_y = INTEGER(y);
 #pragma omp parallel for simd num_threads(n_cores) if (n_cores > 1)
     for (R_xlen_t i = 0; i < xn; ++i) {
@@ -398,9 +443,9 @@ SEXP cpp_set_multiply(SEXP x, SEXP y){
   }
   case REALSXP: {
     copy_warning();
-    Rf_protect(x = Rf_coerceVector(x, REALSXP));
+    Rf_protect(out = Rf_coerceVector(out, REALSXP));
     ++n_protections;
-    double *p_x = REAL(x);
+    double *p_x = REAL(out);
     double *p_y = REAL(y);
 #pragma omp parallel for simd num_threads(n_cores) if (n_cores > 1)
     for (R_xlen_t i = 0; i < xn; ++i) {
@@ -419,7 +464,7 @@ SEXP cpp_set_multiply(SEXP x, SEXP y){
   case INTSXP: {
     // Rf_protect(y = Rf_coerceVector(y, REALSXP));
     // ++n_protections;
-    double *p_x = REAL(x);
+    double *p_x = REAL(out);
     int *p_y = INTEGER(y);
 #pragma omp parallel for simd num_threads(n_cores) if (n_cores > 1)
     for (R_xlen_t i = 0; i < xn; ++i) {
@@ -430,7 +475,7 @@ SEXP cpp_set_multiply(SEXP x, SEXP y){
     break;
   }
   case REALSXP: {
-    double *p_x = REAL(x);
+    double *p_x = REAL(out);
     double *p_y = REAL(y);
 #pragma omp parallel for simd num_threads(n_cores) if (n_cores > 1)
     for (R_xlen_t i = 0; i < xn; ++i) {
@@ -444,7 +489,7 @@ SEXP cpp_set_multiply(SEXP x, SEXP y){
   }
   }
   Rf_unprotect(n_protections);
-  return x;
+  return out;
 }
 
 [[cpp11::register]]
@@ -457,12 +502,17 @@ SEXP cpp_set_divide(SEXP x, SEXP y){
   if (yn > xn){
     Rf_error("length(y) must be <= length(x)");
   }
-  if (!Rf_isReal(x)) copy_warning();
-  Rf_protect(x = Rf_coerceVector(x, REALSXP));
+  SEXP out;
+  if (!Rf_isReal(x)){
+    copy_warning();
+    out = Rf_protect(convert_int_to_real(x));
+  } else {
+    out = Rf_protect(check_transform_altrep(x));
+  }
   switch (TYPEOF(y)){
   case LGLSXP:
   case INTSXP: {
-    double *p_x = REAL(x);
+    double *p_x = REAL(out);
     int *p_y = INTEGER(y);
 #pragma omp parallel for simd num_threads(n_cores) if (n_cores > 1)
     for (R_xlen_t i = 0; i < xn; ++i) {
@@ -473,7 +523,7 @@ SEXP cpp_set_divide(SEXP x, SEXP y){
     break;
   }
   case REALSXP: {
-    double *p_x = REAL(x);
+    double *p_x = REAL(out);
     double *p_y = REAL(y);
 #pragma omp parallel for simd num_threads(n_cores) if (n_cores > 1)
     for (R_xlen_t i = 0; i < xn; ++i) {
@@ -485,7 +535,7 @@ SEXP cpp_set_divide(SEXP x, SEXP y){
   }
   }
   Rf_unprotect(1);
-  return x;
+  return out;
 }
 
 [[cpp11::register]]
@@ -494,17 +544,20 @@ SEXP cpp_set_pow(SEXP x, SEXP y){
   cpp_check_numeric(y);
   R_xlen_t xn = Rf_xlength(x);
   R_xlen_t yn = Rf_xlength(y);
-  int n_protections = 0;
   int n_cores = xn >= 100000 ? num_cores() : 1;
   if (yn > xn){
     Rf_error("length(y) must be <= length(x)");
   }
-  if (!Rf_isReal(x)) copy_warning();
-  Rf_protect(x = Rf_coerceVector(x, REALSXP));
-  ++n_protections;
+  SEXP out;
+  if (!Rf_isReal(x)){
+    copy_warning();
+    out = Rf_protect(convert_int_to_real(x));
+  } else {
+    out = Rf_protect(check_transform_altrep(x));
+  }
   switch (TYPEOF(y)){
   case INTSXP: {
-    double *p_x = REAL(x);
+    double *p_x = REAL(out);
     int *p_y = INTEGER(y);
 #pragma omp parallel for simd num_threads(n_cores) if (n_cores > 1)
     for (R_xlen_t i = 0; i < xn; ++i) {
@@ -519,7 +572,7 @@ SEXP cpp_set_pow(SEXP x, SEXP y){
     break;
   }
   case REALSXP: {
-    double *p_x = REAL(x);
+    double *p_x = REAL(out);
     double *p_y = REAL(y);
 #pragma omp parallel for simd num_threads(n_cores) if (n_cores > 1)
     for (R_xlen_t i = 0; i < xn; ++i) {
@@ -534,8 +587,8 @@ SEXP cpp_set_pow(SEXP x, SEXP y){
     break;
   }
   }
-  Rf_unprotect(n_protections);
-  return x;
+  Rf_unprotect(1);
+  return out;
 }
 
 [[cpp11::register]]
@@ -548,9 +601,14 @@ SEXP cpp_set_log(SEXP x, SEXP base){
     Rf_error("length(y) must be <= length(x)");
   }
   int n_cores = xn >= 100000 ? num_cores() : 1;
-  if (!Rf_isReal(x)) copy_warning();
-  Rf_protect(x = Rf_coerceVector(x, REALSXP));
-  double *p_x = REAL(x);
+  SEXP out;
+  if (!Rf_isReal(x)){
+    copy_warning();
+    out = Rf_protect(convert_int_to_real(x));
+  } else {
+    out = Rf_protect(check_transform_altrep(x));
+  }
+  double *p_x = REAL(out);
   double *p_base = REAL(base);
   if (n_cores > 1){
     OMP_PARALLEL_FOR_SIMD
@@ -568,24 +626,25 @@ SEXP cpp_set_log(SEXP x, SEXP base){
     }
   }
   Rf_unprotect(1);
-  return x;
+  return out;
 }
 
 [[cpp11::register]]
 SEXP cpp_set_round(SEXP x, SEXP digits){
   cpp_check_numeric(x);
   cpp_check_numeric(digits);
-  R_xlen_t xn = Rf_xlength(x);
+  SEXP out = Rf_protect(check_transform_altrep(x));
+  R_xlen_t xn = Rf_xlength(out);
   R_xlen_t digitsn = Rf_xlength(digits);
   if (digitsn > xn){
     Rf_error("length(y) must be <= length(x)");
   }
   int n_cores = xn >= 100000 ? num_cores() : 1;
   // We don't need to round integers.
-  if (Rf_isReal(x)){
+  if (Rf_isReal(out)){
     switch (TYPEOF(digits)){
     case INTSXP: {
-      double *p_x = REAL(x);
+      double *p_x = REAL(out);
       int *p_digits = INTEGER(digits);
       if (n_cores > 1){
         OMP_PARALLEL_FOR_SIMD
@@ -623,7 +682,7 @@ SEXP cpp_set_round(SEXP x, SEXP digits){
       break;
     }
     default: {
-      double *p_x = REAL(x);
+      double *p_x = REAL(out);
       double *p_digits = REAL(digits);
       if (n_cores > 1){
         OMP_PARALLEL_FOR_SIMD
@@ -662,5 +721,6 @@ SEXP cpp_set_round(SEXP x, SEXP digits){
     }
     }
   }
-  return x;
+  Rf_unprotect(1);
+  return out;
 }
